@@ -163,6 +163,8 @@ def delete_nodemap(tr, repo, revlog):
 
 def persist_nodemap(tr, revlog, pending=False, force=False):
     """Write nodemap data on disk for a given revlog"""
+    if len(revlog.index) <= 0:
+        return
     if getattr(revlog, 'filteredrevs', ()):
         raise error.ProgrammingError(
             "cannot persist nodemap of a filtered changelog"
@@ -174,9 +176,9 @@ def persist_nodemap(tr, revlog, pending=False, force=False):
             msg = "calling persist nodemap on a revlog without the feature enabled"
             raise error.ProgrammingError(msg)
 
-    can_incremental = util.safehasattr(revlog.index, "nodemap_data_incremental")
+    can_incremental = hasattr(revlog.index, "nodemap_data_incremental")
     ondisk_docket = revlog._nodemap_docket
-    feed_data = util.safehasattr(revlog.index, "update_nodemap_data")
+    feed_data = hasattr(revlog.index, "update_nodemap_data")
     use_mmap = revlog.opener.options.get(b"persistent-nodemap.mmap")
 
     data = None
@@ -204,11 +206,11 @@ def persist_nodemap(tr, revlog, pending=False, force=False):
                 fd.write(data)
                 if feed_data:
                     if use_mmap:
-                        fd.seek(0)
-                        new_data = fd.read(new_length)
-                    else:
                         fd.flush()
                         new_data = util.buffer(util.mmapread(fd, new_length))
+                    else:
+                        fd.seek(0)
+                        new_data = fd.read(new_length)
             target_docket.data_length = new_length
             target_docket.data_unused = new_unused
 
@@ -216,7 +218,7 @@ def persist_nodemap(tr, revlog, pending=False, force=False):
         # otherwise fallback to a full new export
         target_docket = NodeMapDocket()
         datafile = _rawdata_filepath(revlog, target_docket)
-        if util.safehasattr(revlog.index, "nodemap_data_all"):
+        if hasattr(revlog.index, "nodemap_data_all"):
             data = revlog.index.nodemap_data_all()
         else:
             data = persistent_data(revlog.index)
@@ -611,10 +613,10 @@ def parse_data(data):
 def check_data(ui, index, data):
     """verify that the provided nodemap data are valid for the given idex"""
     ret = 0
-    ui.status((b"revision in index:   %d\n") % len(index))
+    ui.status((b"revisions in index:   %d\n") % len(index))
     root, __ = parse_data(data)
     all_revs = set(_all_revisions(root))
-    ui.status((b"revision in nodemap: %d\n") % len(all_revs))
+    ui.status((b"revisions in nodemap: %d\n") % len(all_revs))
     for r in range(len(index)):
         if r not in all_revs:
             msg = b"  revision missing from nodemap: %d\n" % r
@@ -637,7 +639,7 @@ def check_data(ui, index, data):
 
     if all_revs:
         for r in sorted(all_revs):
-            msg = b"  extra revision in  nodemap: %d\n" % r
+            msg = b"  extra revisions in  nodemap: %d\n" % r
             ui.write_err(msg)
         ret = 1
     return ret
